#!/usr/bin/env node
"use strict";
exports.__esModule = true;
var crypto_1 = require("crypto");
var fs_1 = require("fs");
var path_1 = require("path");
var yargs = require("yargs");
var index_1 = require("./index");
if (require.main === module) {
    yargs
        .usage("$0", "Typescript Codegen", function (y) { return y
        .option("expectNoChanges", {
        boolean: true,
        describe: "Expect No Changes"
    })
        .option("files", {
        array: true,
        demandOption: true,
        describe: "Files"
    }); }, function (argv) {
        argv.files.forEach(function (path) {
            codegenFile(path, argv.expectNoChanges);
        });
    })
        .help()
        .argv;
}
function codegenFile(path, expectNoChanges) {
    var file;
    file = require(process.cwd() + "/" + path);
    for (var name_1 in file) {
        if (file[name_1] instanceof index_1.Module) {
            codegenModule(file[name_1], path, name_1, expectNoChanges);
        }
    }
}
exports.codegenFile = codegenFile;
function codegenModule(module, path, name, expectNoChanges) {
    var dirBuilder = "";
    path_1.dirname(module.destination())
        .split(path_1.sep)
        .forEach(function (dirPart, index) {
        dirBuilder += "" + dirPart + path_1.sep;
        if (!fs_1.existsSync(dirBuilder)) {
            fs_1.mkdirSync(dirBuilder);
        }
    });
    var builder = index_1.Builder["new"]({
        name: name,
        path: path
    });
    var content = module.print(builder);
    codegenModuleWithBespokes(builder.getBespokes(), module.destination(), content, expectNoChanges);
}
exports.codegenModule = codegenModule;
function codegenModuleWithBespokes(bespokes, destination, module, expectNoChanges) {
    var mutableModule = module;
    var originalModule = "";
    if (fs_1.existsSync(destination)) {
        originalModule = fs_1.readFileSync(destination, "ascii");
        if (moduleCodegenIsInvalid(originalModule)) {
            throw new Error("Invalid signature " + destination);
        }
        bespokes.forEach(function (bespoke) {
            mutableModule = codegenModuleWithBespoke(bespoke, mutableModule, originalModule);
        });
    }
    if (expectNoChanges && originalModule !== mutableModule) {
        throw new Error("Codegen outdated " + destination);
    }
    else {
        fs_1.writeFileSync(destination, mutableModule);
    }
}
exports.codegenModuleWithBespokes = codegenModuleWithBespokes;
function moduleCodegenIsInvalid(module) {
    var regexResult = /SIGNED<<(.*)>>/.exec(module);
    if (regexResult === null) {
        return true;
    }
    var actualSignature = regexResult[1];
    var oldLines = module
        .split("\n")
        .slice(8);
    var newLines = [];
    var exclude = false;
    for (var _i = 0, oldLines_1 = oldLines; _i < oldLines_1.length; _i++) {
        var oldLine = oldLines_1[_i];
        if (oldLine.indexOf("BESPOKE END") !== -1) {
            exclude = false;
        }
        if (exclude) {
            continue;
        }
        if (oldLine.indexOf("BESPOKE START") !== -1) {
            exclude = true;
        }
        newLines = newLines.concat([oldLine]);
    }
    var hash = crypto_1.createHash("SHA512");
    hash.update(newLines.join("\n"));
    var expectedSignature = hash.digest("base64");
    return actualSignature !== expectedSignature;
}
exports.moduleCodegenIsInvalid = moduleCodegenIsInvalid;
function codegenModuleWithBespoke(bespoke, module, originalModule) {
    var start = index_1.startTemplate.replace("@0", bespoke);
    var end = index_1.endTemplate.replace("@0", bespoke);
    var originalLines = originalModule.split("\n");
    var originalStartIdx = originalLines.findIndex(function (line) { return line.indexOf(start) !== -1; });
    var originalEndIdx = originalLines.findIndex(function (line) { return line.indexOf(end) !== -1; });
    if (originalStartIdx === -1 || originalEndIdx === -1) {
        return module;
    }
    var customCode = originalLines.slice(originalStartIdx + 1, originalEndIdx);
    var lines = module.split("\n");
    var startIdx = lines.findIndex(function (line) { return line.indexOf(start) !== -1; });
    var endIdx = lines.findIndex(function (line) { return line.indexOf(end) !== -1; });
    var moduleStart = lines.slice(0, startIdx + 1);
    var moduleEnd = lines.slice(endIdx);
    return []
        .concat(moduleStart, customCode, moduleEnd)
        .join("\n");
}
exports.codegenModuleWithBespoke = codegenModuleWithBespoke;
